#!/usr/bin/python
# -*- coding: utf-8 -*-
#"""
#KOPR 1.0
#"""

from math import *
import numpy
import koprhand
import datetime
import datavar

class comet():
    # comet complete object data
    
    def __init__(self, typ='C', destig = '', name = '', elements = {}, perihelions =[], magparams = [] ):
        self.typ = typ # P or C
        self.destig = destig
        self.name = name
        self.elements = elements
        self.perihelions = perihelions
        self.magparams = magparams

    def FindLimits(self, peridate):
        for key in list(self.elements.keys()):
            if key.startswith(peridate):
                tdate = key
        return tdate

    def ReadMPCline(self, line, destig = ''):
        name = line[102:159].strip() 
        perihelion = '/'.join(line[14:29].split())
        output = line[:102] + self.name
        self.elements[perihelion] = output
        rest = line[90:102].split()
        return float(rest[0]), float(rest[1])# Magnitude parameters
    
    def LoadComet(self, destig):
        # Load complete comet info
        self.elements = {}
        self.perihelions = []
        self.magparams = []
        self.destig = destig

        if destig.startswith('P') or destig.startswith('D'):
            self.typ = 'P'
        elif 'P' in destig[:4] or 'D' in destig[:4]:
            self.typ = 'P'
        elif destig.startswith('I') or 'I' in destig[:4]:
            self.typ = 'I'
        elif destig.startswith('A') or 'A' in destig[:4]:
            self.typ = 'A'
        else:
            self.typ = 'C'

        if self.typ == 'P' or self.typ == 'D':
            kinofile = open('./config-files/CometData.dat', 'r')
            for line in kinofile.readlines():
               if koprhand.DestigFromName(line[102:159]) == destig:
                   h0, n = self.ReadMPCline(line)
            kinofile.close() 

            custom_file = open('./config-files/CustomElements.dat', 'r')
            for line in custom_file.readlines():
               if koprhand.DestigFromName(line[102:159]) == destig:
                   self.name = line[102:]
                   h0, n = self.ReadMPCline(line)
            custom_file.close()
            
            orb_file = open('./config-files/Elements.txt', 'r')
            for line in orb_file.readlines():
               if koprhand.DestigFromName(line[102:159]) == destig and len(self.elements) == 0:
                   self.name = line[102:159]
                   h0, n = self.ReadMPCline(line)
               elif koprhand.DestigFromName(line[102:159]) == destig and len(self.elements) > 0:
                  self.name = line[102:159]
                  actualT = line[14:18]
                  for key in list(self.elements.keys()):
                      if actualT == key[:4]:
                          del self.elements[key]
                          self.elements['/'.join(line[14:29].split())] = line
                          rest = line[90:102].split()
                          h0, n = float(rest[0]), float(rest[1])# Magnitude parameters
            orb_file.close()
        else:
            orb_file = open('./config-files/Elements.txt', 'r')
            for line in orb_file.readlines():
               if koprhand.DestigFromName(line[102:159]) == destig:
                  self.name = line[102:159]
                  h0, n = self.ReadMPCline(line)
            if len(self.elements) == 0:
              custom_file = open('./config-files/CustomElements.dat', 'r')
              for line in custom_file.readlines():
                 if koprhand.DestigFromName(line[102:159]) == destig:
                    self.name = line[102:]
                    h0, n = self.ReadMPCline(line,destig)
              custom_file.close()

        for key in list(self.elements.keys()):
            self.perihelions.append(key)
        
        self.magparams = []
        magparamfile = open('./config-files/MagParams.dat', 'r')
        for line in magparamfile.readlines():
            if line.startswith(destig):
                self.magparams.append(line.replace('\n',''))
        magparamfile.close()
        
        try:
          if self.magparams == [] and h0:
            self.magparams.append(','.join([self.destig,str(h0),str(n),'inf','inf']))
        except:
          pass

    def FindElements(self, datedd): # find closest perihelion date
        obstime = koprhand.timecvt(datedd)
        distances, distkeys, distdeltad = [], {}, {}

        for key in list(self.elements.keys()):
            tdate = koprhand.timecvt(key)
            deltadate = float((obstime - tdate).days) + float((obstime - tdate).seconds)/86400
            distances.append(abs(deltadate))
            distkeys[abs(deltadate)] = key
            distdeltad[abs(deltadate)] = deltadate

        outputele = self.elements[distkeys[min(distances)]]

        deltat = distdeltad[min(distances)]
        peridate = distkeys[min(distances)]
        for i in range(len(self.magparams)):
            cname, h0, n, intbeg, intend = self.magparams[i].split(',')
            intbeg = float(intbeg.replace('inf','-9999'))-0.2
            intend = float(intend.replace('inf','9999'))+0.2
            if intbeg <= deltat < intend:
                outputele = outputele[:90] + '{:6.2f}'.format(float(h0)) + ' ' + '{:5.2f} '.format(float(n))+ outputele[102:]
        return outputele, deltat, peridate  

class icqcode():
    # Store an observation on ICQ format
    
    def __init__(self, otype='vis', comet='', date='', time='00:00', extinct=' ', method='', negsign=' ', mag=0, uncsign=' ', magsrc='',\
    apert=0.0, ttype='', focl=0, zoom=0, comasign=' ', coma=0.0, dcsign=' ', dc='  ', tailsign=' ', tail=0, tailpa=0, obscode='',\
    compfl='I ', compint=0, shape='C' , cam='', chip='', sfw='', corr=0, fnumb=0, fcoad='*', error=0, closest='    ', xpix=0.0, ypix=0.0, obsnote='\n', name='' ):
        self.otype = otype # vis or ccd
        self.comet = comet # str
        self.date = date # yyyy/mm/dd
        self.time = time # hh:mm
        self.extinct = extinct # 1str
        self.method = method #
        self.negsign = negsign #
        self.mag = mag #
        self.uncsign = uncsign #
        self.magsrc = magsrc #
        self.apert = apert #
        self.ttype = ttype #
        self.focl = focl #
        self.zoom = zoom #
        self.comasign = comasign #
        self.coma = coma #
        self.dcsign = dcsign #
        self.dc = dc #
        self.tailsign = tailsign #
        self.tail = tail #
        self.tailpa = tailpa #
        self.obscode = obscode #
        # cdd obs only  
        self.compfl = compfl #
        self.compint = compint #
        self.shape = shape #
        self.cam = cam #
        self.chip = chip #
        self.sfw = sfw #
        self.corr = corr #
        self.fnumb = fnumb #
        self.fcoad = fcoad #
        self.error = error #
        self.closest = closest #
        self.xpix = xpix #
        self.ypix = ypix #
        # note
        self.obsnote = obsnote #
        self.name = name

    def decode_icq(self, line):
        if line:
          # Decode a ICQ line
          self.comet = line[0:11] # str
          self.date = line[11:24].replace(' ','/') # yyyy/mm/dd
          #self.time = str('{:2d}:{:2d}'.format(int(float(line[21:24])*24),int( (float(line[21:24])*24 - int(float(line[21:24])*24))*60 ))) # hh:mm
          self.extinct = line[25:26] # 1str
          self.method = line[26:27] #
          self.negsign = line[27:28] #
          self.mag = float(line[28:32]) #
          self.uncsign = line[32:33] #
          self.magsrc = line[33:35] #
          self.apert = line[36:40] #
          self.ttype = line[40:41] #
          self.focl = line[41:43] #
          self.zoom = line[43:47] #
          self.comasign = line[48:49] #
          self.coma = line[49:54] #
          self.dcsign = line[54:55] #
          self.dc = line[55:57] #
          self.tailsign = line[57:58] #
          self.tail = line[58:64] #
          self.tailpa = line[64:67] #
          self.obscode = line[75:80] #
          # cdd obs only 
          if line[68:74].startswith('MPC'):
            if line[68:74].split()[1] == 'Tx':
              self.otype = 'mpct'
            elif line[68:74].split()[1] == 'Nx':
              self.otype = 'mpcn'
          elif line[80:130].strip() != '' or (not self.zoom[0:1].isdigit() and self.zoom[0:1] != ' '):
            self.otype = 'ccd'
            self.compfl = line[80:82] #
            self.compint = line[82:86] #
            self.shape = line[86:877] #
            self.cam = line[93:96] #
            self.chip = line[97:100] #
            self.sfw = line[101:104] #
            self.corr = line[105:106] #
            self.fnumb = line[108:109] #
            self.fcoad = line[109:110] #
            self.error = line[110:114] #
            self.closest = line[115:119] #
            if line[124:125] == 's' and line[120:124].strip() != '' and line[125:129].strip() != '':
                self.xpix = float(line[120:124]) #
                self.ypix = float(line[125:129]) #
            elif line[124:125] == 'm' and line[120:129].strip() != '' and line[125:129].strip() != '':
                self.xpix = float(line[120:124])/60 #
                self.ypix = float(line[125:129])/60 #
            elif line[124:125] == 'd' and line[120:129].strip() != '' and line[125:129].strip() != '':
                self.xpix = float(line[120:124])/3600 #
                self.ypix = float(line[125:129])/3600 #
          else:
            self.otype = 'vis'
          # note
          self.obsnote = line[130:] #

    def build_icq(self):
        if self.otype == 'mpcn' or self.otype == 'mpct':
           zoom = '    '
           extended = '                                                  '

        # Build ICQ code from values
        if ('P' in self.comet[:4]) and ('P' not in self.comet[0:1]): # name transcription
           number,rest = self.comet.split('P',1)
           if int(number)<10:
              identifier = '  '+number+'        '
           elif int(number)<100 and int(number)>=10:
              identifier = ' '+number+'        '
           else:
              identifier = number+'        '
        elif ('P-' in self.comet) and ('P' not in self.comet[0:1]):
           number,rest = self.comet.split('P-',1)
           component,rest = rest.split('/',1)
           if int(number)<10:
              identifier = '  '+number+'       '+component
           elif int(number)<100 and int(number)>=10:
              identifier = ' '+number+'       '+component
           else:
              identifier = number+'       '+component

        elif ('I' in self.comet[:4]) and ('I' not in self.comet[0:1]):
           number,rest = self.comet.split('I',1)
           if int(number)<10:
              identifier = '  '+number+'I       '
           elif int(number)<100 and int(number)>=10:
              identifier = ' '+number+'I       '
           else:
              identifier = number+'I       '
        elif ('I-' in self.comet) and ('I' not in self.comet[0:1]):
           number,rest = self.comet.split('I-',1)
           component,rest = rest.split('/',1)
           if int(number)<10:
              identifier = '  '+number+'I      '+component
           elif int(number)<100 and int(number)>=10:
              identifier = ' '+number+'I      '+component
           else:
              identifier = number+'I      '+component

        else:
           prefix, rest = self.comet.split('/',1)
           if prefix == 'C':
               prefix = ' '
           elif prefix == 'A':
               prefix = 'A'               
           discy, discid = rest.split(' ',1)
           identifier = '  '+prefix+discy+discid
           if len(identifier) == 9:
              identifier = identifier + '  '
           elif len(identifier) == 10:
              identifier = identifier + ' '
           elif len(identifier) == 12:
              num = int(identifier[-3:-1])
              numlast = identifier[-1:]
              char = chr(num - 10 + ord('A')) 
              identifier = identifier[:-3] + char + numlast
        
        self.name = koprhand.CometName(self.comet)
        year, month = self.date.split('/')[0], self.date.split('/')[1] #formatting date
        day = float(self.date.split('/')[2])+ float(self.time.split(':')[0])/24 + float(self.time.split(':')[1])/1440
        datecor = datetime.datetime(int(year), int(month), int(day)) + datetime.timedelta(days=round(day-int(day),2)) # date correction
        year, month, day = '{:d}'.format(datecor.year), '{:02d}'.format(datecor.month), datecor.day + datecor.hour/24. + datecor.minute/1440. + datecor.second/86400.

        if self.focl == 0:
            focl = '  '
        else:
            focl = '{:2d}'.format(int(self.focl))    

        if '.' not in str(self.coma): #formatting coma
           coma = '{:2d}  '.format(int(self.coma))
        else:
           coma = '{:4.1f}'.format(float(self.coma))
    
        if '.' not in str(self.coma): #formatting aperture
           aprt = '{:2d}   '.format(int(self.coma))
        else:
           aprt = '{:5.2f}'.format(float(self.coma))
    
        if self.tail == 0: #formatting tail
           end = '         ICQ XX {:5s}'.format(self.obscode)
        elif self.tail < 0.5:
           end = '{:4.1f}m{:3d} ICQ XX {:5s}'.format(round(self.tail*60,1),self.tailpa,self.obscode)
        else:
           end = '{:3.1f}  {:3d} ICQ XX {:5s}'.format(round(self.tail,1),self.tailpa,self.obscode)

        if self.otype == 'vis' and self.zoom != 0: #formatting visua vs ccd case        
            zoom = ' {:3d}'.format(self.zoom)
            extended = '                                                  '
        elif self.otype == 'vis' and self.zoom == 0:        
            zoom = '    '
            extended = '                                                  '
        elif self.otype == 'ccd': # ccd case start
            if self.zoom >= 1000:
                zoom = '{:1s}{:3s}'.format(chr(int(self.zoom/1000) - 1 + ord('A')),str(self.zoom)[-3:])
            elif self.zoom < 1000:
                zoom = '{:1s}{:3d}'.format('a',self.zoom)

            if self.compint >= 1000:
                compint = '{:1s}{:3s}'.format(chr(int(self.compint/1000) - 1 + ord('A')),str(self.compint)[-3:])
            elif self.compint < 1000 and self.compint > 0:
                compint = '{:1s}{:3d}'.format('a',self.compint)
            elif self.compint == 0:
                compint = '    '

            if self.fnumb > 9:
                self.fnumb = 9

            if self.xpix > 60 or self.ypix > 60:
                pixel = '{:4.1f}m{:4.1f} '.format(self.xpix/60, self.ypix/60)
            else:
                pixel = '{:4.1f}s{:4.1f} '.format(self.xpix, self.ypix)

            extended = '{:2s}{:4s}{:1s}{:4s}m{:3s} {:3s} {:3s} {:1d} U{:1d}*{:3.2f} {:4s} {:s}'.\
            format(self.compfl, compint, self.shape, aprt, self.cam, self.chip, self.sfw, self.corr, self.fnumb, round(self.error,2), self.closest, pixel) 
        elif self.otype == 'mpcn': # mpc case start
            end = '         MPC Nx {:5s}'.format(self.obscode+'  ')
        elif self.otype == 'mpct': 
            end = '         MPC Tx {:5s}'.format(self.obscode+'  ')
        
        basicdest = '{:11s}{:4s} {:2s} {:0>5.2f} '.format(identifier, year, month, round(day,2))
        basicmag = '{:1s}{:1s}{:1s}{:4.1f}{:1s}{:2s} '.format(self.extinct, self.method, self.negsign, round(self.mag,1), self.uncsign, self.magsrc)
        basictel = '{:4.1f}{:1s}{:2s}{:4s} '.format(round(self.apert/10,1), self.ttype, focl, zoom)
        basiccoma = '{:1s}{:4s}  {:2s} {:1s}{:s}'.format(self.comasign, coma, self.dc, self.tailsign, end)

        dc, tail =  '', '.'
        if self.dc != '  ':
            dc = ', DC=' + self.dc 
        if self.tail > 0.5:
            tail = ', Tail: '+ str(self.tail) +' deg. in p.a. ' + str(self.tailpa) + ' deg.'
        elif 0 < self.tail < 0.5:
            tail = ', Tail: '+ str(round(self.tail*60,1)) +' min. in p.a. ' + str(self.tailpa) + ' deg.'
        
        # morris format
        magnote = '' 
        if self.otype == 'ccd':
            morzoom = ''
            for item in ['C','B','V','R','I','g\'','r\'','i\'']:
              if datavar.GetSunMag(item)[1] == self.method:
                magnote = ' CCD-'+item
        else:
            morzoom = '({:d}x)'.format(self.zoom)    
        
        telname = ''
        if self.ttype == 'E':
            telname = 'Naked Eye'
        else:
          telescopes = datavar.Telescopes()
          for key in list(telescopes.keys()):
            if telescopes[key] == self.ttype:
                telname = key
        
        month = month.replace('/01/',' Jan. ').replace('02',' Feb. ').replace('03',' Mar. ').replace('04',' Apr. ').replace('05',' May ').replace('06',' Jun. ')\
                 .replace('07',' Jul. ').replace('08',' Aug. ').replace('09',' Sep. ').replace('10',' Oct. ').replace('11',' Nov. ').replace('12',' Dec. ')
        date = '{:4s} {:2s} {:0>5.2f} '.format(year, month, round(day,2))   
        outputmorris = '{:s}:\n{:s} UT: m1={:4.1f}{:s}, Dia.={:.1f}\'{:s}, {:4.1f}-cm {:s} {:s}{:s}\n[{:s}]'.format( self.name, date, self.mag, magnote, self.coma, dc, round(self.apert/10,1), telname, morzoom, tail, self.obsnote )

        return basicdest + basicmag + basictel + basiccoma + extended + ';' + self.obsnote, outputmorris            

    def get_zoom(self): # translate zoom
        zoom = self.zoom.replace('a','') 
        try:
          zoom = float(zoom)
        except ValueError:
          try:
            zoom = float(zoom[1:]) + float(ord(zoom[0:1]) - ord('A') + 1)*1000.
          except ValueError:
            zoom = 0.

        return zoom
        
    def get_coma(self): # translate coma
        coma = self.coma.replace('&','').replace('<','').replace('>','').replace('!','').replace(' ','')
        if not coma:
          coma = 0.
        else:
          coma = float(coma)
        
        return coma
