#!/usr/bin/python
# -*- coding: utf-8 -*-
#"""
#KOPR 1.0
#"""

import os
import ssl
from math import *
import numpy as np
import alb3
import urllib.request, urllib.error, urllib.parse
import comephem
import datavar
import datetime
import koprhand
import koprclass
import koprdial
import offline
#from astropy.io import fits
from astropy.wcs import WCS

ctx = ssl.create_default_context()
ctx.check_hostname = False
ctx.verify_mode = ssl.CERT_NONE

def PixToWorld(header, x, y):
    proj = WCS(header)
    ra, de = proj.wcs_pix2world( x, y, 1 )
    return float(ra), float(de)

def WorldtoPix(header, ra, de):
    proj = WCS(header)
    x, y = proj.wcs_world2pix( ra, de, 1 )
    return float(x), float(y)

def CalcTailpa(header, x, y, xt, yt):
    pixel, pixelt = (x,y), (xt,yt)
    world, worldt = PixToWorld(header, x, y), PixToWorld(header, xt, yt)
    ra1, ra2, de1, de2 = radians(world[0]), radians(worldt[0]), radians(world[1]), radians(worldt[1])
    dra, dde = ra2 - ra1, de2 - de1
    distance = acos(sin(de1)*sin(de2)+cos(de1)*cos(de2)*cos(dra))
    pa = atan2(sin(dra), cos(de1)*tan(de2)-sin(de1)*cos(dra))
    if pa < 0:
      pa += 2.*pi
    return degrees(distance), degrees(pa)

def CometObsInf(destig, date, time, lon, lat, elevation):
    lon, lat = float(lon), float(lat)
        
    moonaaz, moonalt, mphase = comephem.calcMoon(date,time,lon,lat)
    moonalt, mphase = round(moonalt,1), round(mphase,2)
    
    sunaz, sunalt = comephem.calcSun(date,time,lon,lat)
    sunalt = round(sunalt,1)
    
    comet = comephem.ephem()
    try:
        orb_file = open('./config-files/Elements.txt', 'r')
        for line in orb_file.readlines():
          if koprhand.DestigFromName(line[102:159]) == destig:
              comet.readdb(line)
              comet.compute(date,time,['comet','extend'],lon,lat)              
        orb_file.close()
        orb_file = open('./config-files/CustomElements.dat', 'r')
        for line in orb_file.readlines():
          if koprhand.DestigFromName(line[102:159]) == destig:
              comet.readdb(line)
              comet.compute(date,time,['comet','extend'],lon,lat)              
        orb_file.close()
    except ValueError:
        pass
    return comet, moonalt, mphase, sunalt

def UpdateComParam():
    comets = []
    xlist, results, stdevs = [ [], [], [], [], [], [], [] ], {}, []
    orb_file = open('./config-files/Elements.txt', 'r')
    elines = orb_file.readlines()
    orb_file.close()

    comet_file = open('./config-files/comet-cobs.txt', 'r')
    obslist = comet_file.readlines()
    comet_file.close()

    for line in obslist: # build list of analysed comets
        if line.split(',',1)[0] not in comets:
            comets.append(line.split(',',1)[0])

    newelines = ''
    elements = comephem.ephem()
    for k in range(len(comets)):
        if not comets[k][0:1].isdigit():
            cname = comets[k].split()[0]+' '+comets[k].split()[1]
        else:
            cname = comets[k]
        for line in elines: #load comet
            if line[102:159].startswith(cname):
                eline = line
                elements.readdb(eline)

                for line in obslist: #analyse observations
                    if line.split(',',1)[0] == comets[k]:
                        comet, cid, dtime, mag = line.split(',')
                        elements.compute(dtime, '00:00', ['comet'] )
                        try:
                          for j in range(7):
                            xlist[j].append(float(mag)-5*log10(elements.dist) - (j+2.)*2.5*log10(elements.r))
                        except:
                          pass

                if len(xlist[0]) < 3:
                  intercept, slope = np.mean(xlist[2]), 4.
                else:
                  for j in range(7): # find best parameters
                    stdev = np.std(xlist[j])
                    stdevs.append(stdev) 
                    results[stdev] = [ np.mean(xlist[j]) , j+2. ]
                  intercept, slope = results[min(stdevs)]
                xlist, results, stdevs = [ [], [], [], [], [], [], [] ], {}, []    

                newline = eline[:90] + '{:6.2f}'.format(round(intercept,2)) + '{:5.2f} '.format(round(slope,2))+ eline[102:] 
                newelines = newelines + newline

    output = ''

    for line in elines:
        cname = line[102:159].strip()
        if not cname[0:1].isdigit():
            cname = cname.split()[0]+' '+cname.split()[1]
        else:
            cname = cname
        if not any(item.startswith(cname) for item in comets):
            output = output + line
    
    output = output + newelines

    orb_file = open('./config-files/Elements.txt', 'w')                
    orb_file.write(output)
    orb_file.close()

def extcalc(extinction,alth,elevation):
    if extinction == 'Winter':
        extsign, extinct = 'w', 0.035
    elif extinction == 'Summer':
        extsign, extinct = 's', 0.065
    else:
        extsign, extinct = 'a', 0.050
    return alb3.extcalc(extinct,float(alth),float(elevation)), extsign

def CalcMag(date, eline, h0 = 0, n = 0, distcorr = 0):
    elements = comephem.ephem()
    elements.readdb(eline)
    if h0 != 0:
        elements.h0 = h0
        elements.n = n 
    elements.compute(str(date.strftime('%Y/%m/%d')),str(date.strftime('%H:%M')), ['comet'])
    if distcorr == 1:
        return elements.mag - 5. * log10(elements.dist)
    else: 
        return elements.mag

def MagCorrs2(mag, phasea, d2g, corrph, corrfwsc):
    phasemag, fwscmag = 0., 0.
    if corrph == 1:
        phasemag = alb3.phasemag(phasea)   
    if corrfwsc == 1:
        gf, gb, kk = 0.9, -0.6, 0.95
        fwscmag = alb3.fwscmag(phasea, d2g, gf, gb, kk)
    return mag - phasemag + fwscmag

def Centroid(image_data,x0,y0,sky,r = 15):
    data = []
    counts = []
    for x in range (int(x0-r-1),int(x0+r+2)):
        for y in range(int(y0-r-1),int(y0+r+2)):
            c, xs, ys = alb3.centr(x,x0,y,y0)
            if c < (r + 1):
                data.append([x,y,image_data[y,x] - sky])
    threshold = np.percentile([i[2] for i in data],99)
    data = [[xy[0],xy[1],0] if xy[2] < threshold else xy for xy in data]
    values = np.array(data)
    center = np.average(values[:,:2], axis=0, weights=values[:,2])
    return center[0], center[1]

def ImageStats(image_data):
    threshold = np.percentile(image_data, 60)
    mnorm2d = np.ma.masked_greater(image_data,threshold)
    return np.mean(mnorm2d), np.amax(image_data), np.percentile(image_data, 5), np.percentile(image_data, 99), np.median(image_data)

def BackgroundSky(image_data, x, y, rad):
    a, x, y = np.copy(image_data), int(x), int(y)
    dist1 = int(rad/3. + rad)
    dist2 = int(rad/3. + rad*2.)
    inn = [ y - dist1, y + dist1 + 1, x - dist1, x + dist1 + 1 ]#boundaries
    out = [ y - dist2, y + dist2 + 1, x - dist2, x + dist2 + 1 ]
    for subs in [out,inn]:#get rid of values under 0
      for index, item in enumerate(subs):
        if item < 0:
          subs[index] = 0
    a[inn[0]:inn[1],inn[2]:inn[3]] = 0 #null inner subsection
    d = a[out[0]:out[1],out[2]:out[3]] #slice array
    b = image_data[out[0]:out[1],out[2]:out[3]]
 
    return np.ma.median(np.ma.masked_greater(np.ma.masked_equal(d,0),np.percentile(b,60))), (inn[2],inn[0]), (out[2],out[0])

def readstar(data):
    output = []
    for x in data:
      try: 
        output.append(float(x))
      except ValueError:
        output.append('n/a')

    return output  

def catCheck(directory):
    if os.path.exists(directory):
      for k in range(-18,18):
        if k < 0:
          prefix = 'zm'
        else: 
          prefix = 'zp'
        filetest = directory+(prefix + str(k*5) + '.txt').replace('-','')
        if os.path.isfile(filetest):
          pass
        else:
          print('Missing file: ' + directory+filetest)
          return False
      return True 

def StarData(header, maglim, magsrc):
    # check if exist local data files
    if catCheck('./16-koprdr10/'):
      print('local APASS 16 data found')
      return offline.GetLocalApass(header, maglim, magsrc, './16-koprdr10/')
    elif catCheck('./14-koprdr10/'):
      print('local APASS 14 data found')
      return offline.GetLocalApass(header, maglim, magsrc, './14-koprdr10/')
    else: 
      # else get from vizier
      return VizierStars(header, maglim, magsrc)

def VizierStars(header, maglim, magsrc):  
    magsrc, vizstr = datavar.MagSources()[magsrc][0].split(), datavar.MagSources()[magsrc][1]
    params = koprhand.SettingsRead()
    
    if magsrc[1] == 'calc.' and magsrc[2] == 'R':
      plen = 6
    elif magsrc[1] == 'calc.' and magsrc[2] == 'I':
      plen = 7
    else:
      plen = 5

    xdim, ydim, ra, de = header['CRPIX1']*2., header['CRPIX2']*2., header['CRVAL1'], header['CRVAL2'] 
    try: 
      fov = header['CDELT1']*xdim
    except KeyError:
      cdelt1, cdelt2 = PixToWorld(header, 0, 0), PixToWorld(header, xdim, ydim)
      fov = sqrt((cdelt1[0]-cdelt2[0])*(cdelt1[0]-cdelt2[0]) + (cdelt1[1]-cdelt2[1])*(cdelt1[1]-cdelt2[1]))

    if abs(fov) > float(params[1][8]) and params[1][8] != '0': # Max FOV applied
      fov = float(params[1][8])
    
    starlist, blustar, yelstar, redstar, nonestar = [], [], [], [], []
    #r    _RAJ       _DEJ       B-V    mag1     mag2   mag3     # 0     1          2         3      4       5       6 
    try:
      urltext = 'http://vizier.u-strasbg.fr/viz-bin/asu-txt?-source=II/336/apass9&-c='+str(ra)+','+str(de)+'&-c.bd='+str(fov)+'&-out=_r&-out=_RAJ2000&-out=_DEJ2000&-out=B-V'+vizstr+'&-sort=_r'
      print('URL request: ',urltext)
      response = urllib.request.urlopen(urltext, context=ctx)
      # show dialog with URL
      if params[1][12] == '2':
        koprdial.InfoDialog(0,'Getting stars from URL request: ', '<a href="'+urltext+'">'+urltext+'</a>')
    
      html = response.readlines()
      for line in html:
        line = line.decode('UTF-8')
        try:
          if plen == 5:
            output = ( line[0:8], line[9:21], line[22:34], line[35:41], line[42:48].replace('\n','') )
            starra, starde, starbv, starmag = readstar(output)[1:]
          elif plen == 6:
            output = ( line[0:8], line[9:21], line[22:34], line[35:41], line[42:48], line[49:55].replace('\n','') )
            starra, starde, starbv, starr, stari = readstar(output)[1:]
            try:
              starmag = (-0.153)*( starr - stari ) - ( 0.117 ) + starr
            except TypeError:
              starmag = 50
          elif plen == 7:
            output = ( line[0:8], line[9:21], line[22:34], line[35:41], line[42:48], line[49:55], line[56:62].replace('\n','') )
            starra, starde, starbv, starv, starg, stari = readstar(output)[1:]
            try:
              if (starg-stari) <= 2.1:
                starmag = starv - (0.675)*(starg-stari) - (0.364) 
              else:
                starmag = starv - (1.11)*(starg-stari) + (0.52)
            except TypeError:
              starmag = 50
              
          x, y = WorldtoPix( header, float(starra), float(starde) )
          if starmag != 'n/a':
            if ( 0 < x < xdim ) and ( 0 < y < ydim ) and ( starmag < maglim ) : 
              starlist.append( [x, y, starmag, starbv] )

              if starbv == 'n/a':
                nonestar.append( [x, y] )
              elif starbv < 0.5: 
                blustar.append( [x, y] )
              elif 0.5 <= starbv <= 0.8: 
                yelstar.append( [x, y] )
              elif starbv > 0.8: 
                redstar.append( [x, y] )
        except Exception as ex:
          pass

    except Exception as e:
      if 'Connection timed out' in str(e):
        koprdial.InfoDialog(1,'Connection timed out', 'Connection timed out to http://vizier.u-strasbg.fr')

    return starlist, blustar, yelstar, redstar, nonestar

def readUCAC(ra, de, fov, mapparams):
    output, output0 = [], []
    # check if exist local data files
    if catCheck('./16-koprdr10/'):
      print('local APASS 16 data found')
      output = offline.GetLocalApassVIZ(ra, de, fov, './16-koprdr10/')
    elif catCheck('./14-koprdr10/'):
      print('local APASS 14 data found')
      output = offline.GetLocalApassVIZ(ra, de, fov, './14-koprdr10/')
    else: 
      # else get from vizier
      try:
        response = urllib.request.urlopen('http://vizier.u-strasbg.fr/viz-bin/asu-txt?-source=II/336/apass9&-c='+str(ra)+','+str(de)+'&-c.bd='+str(fov)+'&-out=_r&-out=_RAJ2000&-out=_DEJ2000&-out=Vmag&-out=B-V&-sort=_r', context=ctx)
        response0 = urllib.request.urlopen('http://vizier.u-strasbg.fr/viz-bin/asu-txt?-source=I/322A/out&-c='+str(ra)+','+str(de)+'&-c.bd='+str(fov)+'&-out=_r&-out=_RAJ2000&-out=_DEJ2000&-out=f.mag&-sort=_r', context=ctx)
        print('URL response read')
        output, output0 = response.readlines(), response0.readlines()
        output, output0 = [x.decode('UTF-8') for x in output], [x.decode('UTF-8') for x in output0]
      except Exception as e:
        if 'Connection timed out' in str(e):
          koprdial.InfoDialog(1,'Connection timed out', 'Connection timed out to http://vizier.u-strasbg.fr')

    return [output, mapparams, output0]

def readTYC(ra, de, fov, mapparams):
    output = []
    # check if exist local data files
    if catCheck('./koprtyc2/'):
      print('Local KOPR TYC2 detected')
      output = offline.getofflineTYC(ra, de, fov)
    else:    
      try:
        response = urllib.request.urlopen('http://vizier.u-strasbg.fr/viz-bin/asu-txt?-source=I/259/tyc2&-c='+str(ra)+','+str(de)+'&-c.bd='+str(fov)+'&-out=_r&-out=_RAJ2000&-out=_DEJ2000&-out=VTmag&-out=BTmag&-out=TYC1&-out=TYC2&-out=TYC3&-sort=_r', context=ctx)
        print('URL response read: ','http://vizier.u-strasbg.fr/viz-bin/asu-txt?-source=I/259/tyc2&-c='+str(ra)+','+str(de)+'&-c.bd='+str(fov)+'&-out=_r&-out=_RAJ2000&-out=_DEJ2000&-out=VTmag&-out=BTmag&-out=TYC1&-out=TYC2&-out=TYC3&-sort=_r')
        output = response.readlines()
        output = [x.decode('UTF-8') for x in output]
      except Exception as e:
        if 'Connection timed out' in str(e):
          koprdial.InfoDialog(1,'Connection timed out', 'Connection timed out to http://vizier.u-strasbg.fr')

    return [output, mapparams]

def YaleRaDe(data):
    result = []
    for line in data:
      output = line.split()
      try:
        starra, starde, starmag, starbv, stdest = (float(output[1])+float(output[2])/60.+float(output[3])/3600.)*15., float(output[4])+float(output[5])/60.+float(output[6])/3600., output[7], output[8], output[9]
        result.append(' '.join( [stdest, str(starra), str(starde), starmag, starbv, stdest]  ))
      except:
        pass
    return result

def readYBSC(ra, de, fov, mapparams):
    output = []
    # check if exist local data files V/50 http://vizier.u-strasbg.fr/viz-bin/asu-txt?-source=V/50&-c=45.6,18.8&-c.bd=5.2&-out=_r&-out=RAJ2000&-out=DEJ2000&-out=Vmag&-out=B-V&-out=HR&-sort=_r
    if os.path.isfile('./koprybsc5/bsc5.txt'):
      print('Local KOPR YBSC5 detected')
      output = offline.getofflineYBSC(ra, de, fov)
    else:
      try:
        response = urllib.request.urlopen('http://vizier.u-strasbg.fr/viz-bin/asu-txt?-source=V/50&-c='+str(ra)+','+str(de)+'&-c.bd='+str(fov)+'&-out=_r&-out=RAJ2000&-out=DEJ2000&-out=Vmag&-out=B-V&-out=HR&-sort=_r', context=ctx)
        print('URL response read: ','http://vizier.u-strasbg.fr/viz-bin/asu-txt?-source=V/50&-c='+str(ra)+','+str(de)+'&-c.bd='+str(fov)+'&-out=_r&-out=RAJ2000&-out=DEJ2000&-out=Vmag&-out=B-V&-out=HR&-sort=_r')
        output = response.readlines()
        output = [x.decode('UTF-8') for x in output]
        output = YaleRaDe(output)
      except Exception as e:
        if 'Connection timed out' in str(e):
          koprdial.InfoDialog(1,'Connection timed out', 'Connection timed out to http://vizier.u-strasbg.fr')

    return [output, mapparams]

def GenerateMap(mapparams, vizdata, dra=0, dde=0): #ra, de, fov, maglim = 
    ra, de, fov, maglim, source = mapparams[:5]
    ra, de, fov, maglim = float(ra), float(de), float(fov), float(maglim)
    xdim, ydim, x, output = 1024*2, 768*2, [], []
    
    if dra != 0 and dde != 0:
      ydim = ydim / 2.
    
    pix = fov/xdim
    header  = { 'NAXIS'  : 2,
                'NAXIS1' : xdim,
                'CTYPE1' : 'RA---TAN',
                'CRVAL1' : ra,
                'CRPIX1' : xdim/2,
                'CUNIT1' : 'deg',
                'CDELT1' : -pix,
                'NAXIS2' : ydim,
                'CTYPE2' : 'DEC--TAN',
                'CRVAL2' : de,
                'CRPIX2' : ydim/2,
                'CUNIT2' : 'deg',
                'CDELT2' : -pix, }

    if dra != 0 and dde != 0:
      header['CROTA2'] = degrees(atan2(dde,dra))
        
    if not vizdata:  
        if source == 'TYC-2':
            vizdata = readTYC(ra, de, fov, mapparams) 
        elif source == 'UCAC4':
            vizdata = readUCAC(ra, de, fov, mapparams)
        elif source == 'YBSC5':
            vizdata = readYBSC(ra, de, fov, mapparams)
    elif vizdata and (float(vizdata[1][0]) != ra or float(vizdata[1][1]) != de or float(vizdata[1][2]) < fov or float(vizdata[1][3]) < maglim or vizdata[1][4] != source):
        if source == 'TYC-2':
            vizdata = readTYC(ra, de, fov, mapparams) 
        elif source == 'UCAC4':
            vizdata = readUCAC(ra, de, fov, mapparams)
        elif source == 'YBSC5':
            vizdata = readYBSC(ra, de, fov, mapparams)

    starlist = [[],[]]
    print('size ',len(vizdata[0]))
    for line in vizdata[0]:
        try:
            output = line.split()
            if len(output) > 4:
              if source == 'TYC-2':
                try: # filter uncomplete stars
                  starra, starde, starmag, starbv, stdest = float(output[1]), float(output[2]), float(output[3]), 0.850 * (float(output[4])-float(output[3])), 'TYC '+str(output[5])+'-'+str(output[6])+'-'+str(output[7])
                except:
                  pass # print len(output), output
              elif source == 'YBSC5':
                try: # filter uncomplete stars
                  starra, starde, starmag, starbv, stdest = float(output[1]), float(output[2]), float(output[3]), float(output[4]), 'HR '+str(output[5])
                except:
                  pass # print len(output), output
              elif source == 'UCAC4':
                starra, starde, starmag, starbv, stdest = float(output[1]), float(output[2]), float(output[3]), float(output[4]), 'APASS '+output[0]
              x, y = WorldtoPix( header, starra, starde )
              if ( 0 < x < xdim ) and ( 0 < y < ydim ) and ( starmag < maglim ) : 
                starlist[0].append( [x, y, starmag, stdest, starbv] )
        except Exception as ex:
          pass
#          print(ex)
#          print(line)      

    if len(vizdata) == 3:
      for line in vizdata[2]:
        try:
            output = line.split()
            if len(output) > 4:
              starra, starde, starmag = float(output[1]), float(output[2]), float(output[3])
              x, y = WorldtoPix( header,starra,starde )
              if ( 0 < x < xdim ) and ( 0 < y < ydim ) and ( starmag < maglim ) : 
                starlist[1].append( [x, y, starmag] )
        except Exception as ex:
          pass
#          print(ex)
#          print(line)      
                                    
    return header, starlist, vizdata

def ProcessObs(refstars, extinction, cometalt, elev ):
    xlist, ylist, slist = [], [], []

    for star in refstars:
        if star[0] == 'o': 
            if extinction == 'No Extinction':
                extcorr, comext, extinct = 0, 0, ' '
            else:
                comext, extinct = extcalc(extinction, cometalt, elev)
                extcorr = extcalc( extinction, float(star[4]), elev )[0]
        
            xlist.append( float( star[1] ) )
            ylist.append( float( star[2] ) + extcorr - comext )
            slist.append( star[5] )

    slope, mag = np.polyfit(xlist,ylist,1)

    t = np.arange(min(xlist), max(xlist), 0.02)
    s = list()
    for i in range(len(t)):
        s.append(mag + (min(xlist)+i*0.02) * slope)
    
    return mag, extinct, xlist, ylist, slist, t, s, slope

def CalculateMag(cometrel, stars, extinction, elev):
    xlist, extcorr, comext = [], 0, 0

    if extinction != 'No Extinction':
        comext = extcalc( extinction, cometrel[1], elev)[0]
    
    for star in stars:
        if 'o' in star[0]:
            if extinction != 'No Extinction':
                extcorr  = extcalc( extinction, koprhand.degconvert(str(star[4])), elev )[0]
            xlist.append( star[2] - ( star[1] - cometrel[0] ) + extcorr )

    return np.mean( xlist, axis=0 ) - comext, np.std(xlist), xlist

def GenerateDiag(times, x, y):
    dtimes, init, addon = [ 0 ], times[0], 0
    for i in range(1, len(times)):
        if times[i]+0.1 < times[i-1]:            
            addon = 24.
        dtimes.append( times[i] - times[0] + addon )
    step = 2. / (x*y)
    
    pxhour = x / dtimes[7]

    start, end, delta = int(times[0])+1, int(times[7]), int(times[0]) + 1. - times[0] 
    ticks, add, t = [], 0, start
    while t != end:
      t = t + add
      if t > 23:
          t = t - 24
      ticks.append( (t, int(round(delta*pxhour,0))) )
      t += 1
      delta += 1. 
    ticks.append( (end, int(round(delta*pxhour,0))) )
    
    value, intvl = 0, []
    for i in range(len(times)-1):
        value = value + (dtimes[i+1]-dtimes[i]) * pxhour
        intvl.append(int(round(value,0)))
    
    diag = np.zeros((y, x, 3), dtype = np.float32)
    
    #                  civil                naut               astro              night 
    #colors = [ [ [ 0.8, 0.8, 0.8 ], [ 0.6, 0.6, 0.6 ], [ 0.3, 0.3, 0.3 ], [ 0.0, 0.0, 0.0 ], [ 0.3, 0.3, 0.3 ], [ 0.6, 0.6, 0.6 ], [ 0.8, 0.8, 0.8 ] ], #  down to 30 
    #           [ [ 0.9, 0.8, 0.8 ], [ 0.7, 0.6, 0.6 ], [ 0.4, 0.3, 0.3 ], [ 0.2, 0.0, 0.0 ], [ 0.4, 0.3, 0.3 ], [ 0.7, 0.6, 0.6 ], [ 0.9, 0.8, 0.8 ] ], #  down to 10  
    #           [ [ 1.0, 0.8, 0.8 ], [ 0.8, 0.6, 0.6 ], [ 0.6, 0.3, 0.3 ], [ 0.4, 0.0, 0.0 ], [ 0.6, 0.3, 0.3 ], [ 0.8, 0.6, 0.6 ], [ 1.0, 0.8, 0.8 ] ] ]#  down to horizon

    colors = [ [ [ 0.8, 0.9, 0.9 ], [ 0.8, 0.87, 0.87 ], [ 0.8, 0.83, 0.83 ], [ 0.8, 0.8, 0.8 ], [ 0.8, 0.83, 0.83 ], [ 0.8, 0.87, 0.87 ], [ 0.8, 0.9, 0.9 ] ], #  down to 30 
               [ [ 0.9, 0.9, 0.9 ], [ 0.9, 0.87, 0.87 ], [ 0.9, 0.83, 0.83 ], [ 0.9, 0.8, 0.8 ], [ 0.9, 0.83, 0.83 ], [ 0.9, 0.87, 0.87 ], [ 0.9, 0.9, 0.9 ] ], #  down to 10  
               [ [ 1.0, 0.9, 0.9 ], [ 1.0, 0.87, 0.87 ], [ 1.0, 0.83, 0.83 ], [ 1.0, 0.8, 0.8 ], [ 1.0, 0.83, 0.83 ], [ 1.0, 0.87, 0.87 ], [ 1.0, 0.9, 0.9 ] ] ]#  down to horizon
    
    xc = [ 0, intvl[0], intvl[1], intvl[2], intvl[3], intvl[4], intvl[5], x ]
    yc = [ 0, y - y/9*3, y - y/9, y ]
    
    for xr in range(1,8):
      for yr in range(1,4):
         diag[ int(yc[yr-1]):int(yc[yr]), xc[xr-1]:xc[xr] ] = colors[yr-1][xr-1]

    return diag, ticks  
    
def GenerateTimes( dyear, dmonth, dday, dhour, longn, lat, gmt ):
    ddays = alb3.jday(dyear,dmonth,dday,dhour) - 2451543.5 
    sunalts, times = [ 0.833, 6., 12., 18. ], []
    meanlon, ilonsun, rs, ecl = alb3.sunpos(ddays)
    xs, ys = alb3.suncoords(ilonsun, rs)
    iras, ides = alb3.sunrade(xs, ys, ecl)                 
    lst, mst = alb3.lst(longn,ddays)
    
    for sunalt in sunalts: 
        ras, des, lonsun = iras, ides, ilonsun 
        tz, tv = alb3.riseset(lonsun, ras, des, longn, lat, sunalt )
        for i in range(4):
          meanlon, lonsun, rs, ecl = alb3.sunpos(alb3.jday(dyear,dmonth,dday,tz) - 2451543.5 + 1.)
          xs, ys = alb3.suncoords(lonsun, rs)
          ras, des = alb3.sunrade(xs, ys, ecl) 
          tz = alb3.riseset(lonsun, ras, des, longn, lat, sunalt )[0]
        for i in range(4):
          meanlon, lonsun, rs, ecl = alb3.sunpos(alb3.jday(dyear,dmonth,dday,tv) - 2451543.5)
          xs, ys = alb3.suncoords(lonsun, rs)
          ras, des = alb3.sunrade(xs, ys, ecl) 
          tv = alb3.riseset(lonsun, ras, des, longn, lat, sunalt )[1]
        tz, tv = tz + gmt, tv + gmt

        if tz > 24.:
          tz = tz - 24.
        elif tz < 0:
          tz = tz + 24.
        
        if tv > 24.:
          tv = tv - 24.
        elif tz < 0:
          tv = tv + 24.
          
        times.append( tz )
        times.append( tv )
    
    return [ times[0], times[2], times[4], times[6], times[7], times[5], times[3], times[1] ] 

def DiagPlotList(year, month, day, longn, lat, gmt, times, cometlist, x, y):
  now = datetime.datetime(year, month, day, int(times[0]), int((times[0]-int(times[0]))*60) ) - datetime.timedelta(hours = gmt)
  initlst = alb3.lst( longn, alb3.jday(now.year,now.month,now.day,now.hour + now.minute/60.))[0]

  if times[7] < times[0]:
    dist = (times[7] + 24) - times[0]
  else:
    dist = times[7] - times[0]

  pxtime, pxdeg = dist/x, y/90.
  t, s = [], []
  for i in range(len(cometlist)):
    com, pxs, lst = [], [], initlst  
    for j in range(int(x)):
      lst += pxtime/0.997269567
      alt = alb3.objaltaaz(lst, cometlist[i][1], cometlist[i][2], lat)[1]
      ycoord = (90.-alt)*pxdeg
      if alt > 0:
        pxs.append( j+1 )
        com.append( ycoord )
    t.append(pxs)
    s.append(com)
  return t, s

def MPCComParam(peridate, kometa, obslist, slope, mpct, mpcn):
    eline = kometa.FindElements(peridate)[0]
    elements = comephem.ephem()
    elements.readdb(eline)
    
    fobslist, xlist, xlist0 = [], [], []
    for line in obslist: #analyse observations
      mpcobs = koprclass.icqcode()
      mpcobs.decode_icq(line)
      peri = kometa.FindElements(mpcobs.date)[2]
      if peri[:4] == peridate[:4]: 
        if (mpcobs.otype == 'mpct' and mpct == 1) or (mpcobs.otype == 'mpcn' and mpcn == 1):
          fobslist.append(line)
          elements.compute(mpcobs.date, '00:00', ['comet'] )
          if slope != 0:
            xlist.append(float(mpcobs.mag)-5*log10(elements.dist) - slope*2.5*log10(elements.r))
          else:
            for j in range(2,9):
              xlist.append(float(mpcobs.mag)-5*log10(elements.dist) - j*2.5*log10(elements.r))
    
    if slope != 0:
      intercept = np.mean(xlist)
    else:
      for j in range(7): # find best parameters
        for i in range(int(len(xlist)/7)):
            xlist0.append(xlist[7*i+j])
        intercept0 = np.mean(xlist0)
        std0 = np.std(xlist0)
        if j == 0:
            intercept = intercept0
            std = std0
            slope = float(j+2)
        else:
            if round(std0,2) < round(std,2):
                intercept = intercept0
                std = std0
                slope = float(j+2)
        xlist0 = []

    return intercept, slope, fobslist, eline

def FilterMPCObs(eline, obslist, perc, intercept, slope ):
  deltamag, obsdict, newobs, dates = [], {}, [], []
  elements = comephem.ephem()
  elements.readdb(eline)
  elements.h0, elements.n = intercept, slope
  for obs_line in obslist:
    obs = koprclass.icqcode()
    obs.decode_icq(obs_line)
    elements.compute(obs.date,'00:00',['comet'])
    diff = obs.mag - elements.mag
    deltamag.append(diff)
    dates.append(obs.date)
    obsdict[diff] = obs_line
  
  perc = np.percentile( deltamag, perc )

  for key in list(obsdict.keys()):
    if key < perc:
      newobs.append('o'+obsdict[key])
    else:
      newobs.append('x'+obsdict[key])

  beg, end =  min(dates), max(dates)
   
  diff = koprhand.timecvt(end) - koprhand.timecvt(beg)
  steps = diff.days + 1

  tmake, smake, outobs = [], [], []
   
  t, s = [], []
  for k in range(steps): # generate line
    actualdate = koprhand.timecvt(beg) + datetime.timedelta(days= k)
    elements.compute( koprhand.cvt2time(actualdate),'00:00',['comet'] )
    t.append(actualdate)
    s.append(elements.mag)
  tmake.append(t)
  smake.append(s)
       
  t, s = [], []
  for obs_line in newobs: # generate list of positive obs
    if obs_line.startswith('o'):
      obs = koprclass.icqcode()
      obs.decode_icq(obs_line[1:])
      t.append(koprhand.timecvt(obs.date))
      s.append(obs.mag)
      outobs.append(obs_line[1:])
  tmake.append(t)
  smake.append(s)
    
  t, s = [], []
  for obs_line in newobs: # generate list of negative obs
    if obs_line.startswith('x'):
      obs = koprclass.icqcode()
      obs.decode_icq(obs_line[1:])
      t.append(koprhand.timecvt(obs.date))
      s.append(obs.mag)
  tmake.append(t)
  smake.append(s)

  return outobs, tmake, smake

### to be deleted
def AnalyzeData( obs_list, kometa, perasy, phasec, forwsca, fslope, percoef ):
  elements = comephem.ephem()
  xlist, ylist, deltalist = [], [], []
  
  if percoef == 1:
    personalcoef = koprhand.LoadPersonalCoef()

  for line in obs_list:
    obs = koprclass.icqcode()
    obs.decode_icq(line)
    eline, deltadate, peridate = kometa.FindElements(obs.date)
    elements.readdb(eline)

    if phasec == 1 or forwsca == 1:
      elements.compute(obs.date,'00:00', ['comet','phasec'])
      resultmag = obs.mag + elements.phasec * phasec + elements.forwscac * forwsca
    else:
      elements.compute(obs.date,'00:00', ['comet'])
      resultmag = obs.mag

    if percoef == 1 and obs.obscode in list(personalcoef.keys()) and obs.otype == 'vis': # personal coeff apply
      resultmag = resultmag - personalcoef[obs.obscode]

    if perasy == 1 and deltadate < 0:
      ylist.append(resultmag-5*log10(elements.dist)-2.5*fslope*log10(elements.r))
      xlist.append(log10(elements.r))
      deltalist.append(deltadate)
    elif perasy == 2 and deltadate > 0:
      ylist.append(resultmag-5*log10(elements.dist)-2.5*fslope*log10(elements.r))
      xlist.append(log10(elements.r))
      deltalist.append(deltadate)
    elif perasy == 0:
      ylist.append(resultmag-5*log10(elements.dist)-2.5*fslope*log10(elements.r))
      xlist.append(log10(elements.r))
      deltalist.append(deltadate)

  if fslope != 0:
    h0, n, std = np.mean(ylist), fslope, np.std(xlist)
  else:
    result, std, _, _, _  = np.polyfit(xlist,ylist,1, full=True)   
    n, h0, std = result[0]/2.5, result[1] , float(std)/len(ylist)

  if perasy == 1:
    beg, end = min(deltalist), 0
  elif perasy == 2:
    beg, end = 0, max(deltalist)
  elif perasy == 0:
    beg, end = min(deltalist), max(deltalist)

  return h0, n, std, len(ylist), beg, end
### to be deleted end

def PersonalCoef(obs_list, kometa, phasec, forwsca):
  xlist, elements = [], comephem.ephem()
  
  for line in obs_list:
    obs = koprclass.icqcode()
    obs.decode_icq(line)
    if obs.otype == 'vis':
      eline, deltadate, peridate = kometa.FindElements(obs.date)
      elements.readdb(eline)
      if phasec == 1 or forwsca == 1:
        elements.compute(obs.date,'00:00', ['comet','phasec'])
        resultmag = obs.mag + elements.phasec * phasec + elements.forwscac * forwsca
      else:
        elements.compute(obs.date,'00:00', ['comet'])
        resultmag = obs.mag
      diff = resultmag - elements.mag
      xlist.append( [ obs.obscode, obs.date, obs.comet, str(diff) ] )

  # local data analyze
  observers, results = [], {}
  for line in xlist: # now analyze dataset
      if line[0] not in [item[0] for item in observers]:
          observers.append([line[0],'o'])
  for line in observers:
      line = line[0]
      stats = []
      for data in xlist:
          if line == data[0]:
              stats.append(float(data[3]))
      results[line] = round(np.mean(stats),2)

  return xlist, observers, results, kometa.destig.replace(' ','').replace('/','')
  
def ApplyPersonal(xlist, observers, results, destig):
  comdiff_file = open('./obs-data/'+destig+'.dat','w')
  
  alvd_obs = []
  for obs in observers:
    if obs[1] == 'o':
      alvd_obs.append(obs[0])
  
  for key in list(results.keys()):
    if key in alvd_obs:
      comdiff_file.write(key+'='+str(results[key])+'\n')
  comdiff_file.close()

  # global data update  
  try:
    diff_file = open('./obs-data/diffdata.dat','r') # load actual data
    diffdata = diff_file.readlines()  
    diff_file.close()
  except:
    diffdata = [] 

  newdiff = []
  for line in diffdata: # compare updates
      lst, count = line.replace('\n','').split(','), 0
      for data in xlist:
          if data[0] == lst[0] and data[1] == lst[1]:
              count += 1
      if count == 0 and line != '': 
          newdiff.append(line.replace('\n','')) # add unchanged
  for line in xlist: # add new
    if line[0] in alvd_obs: 
      newdiff.append(','.join(line))     
  diff_file = open('./obs-data/diffdata.dat','w') # save to file
  diff_file.write('\n'.join(newdiff))
  diff_file.close()
  
  # global data analyze
  observers = []
  for line in newdiff: # now analyze all datasets
      if line.split(',')[0] not in observers:
          observers.append(line.split(',')[0])
  results, output = {}, []
  for line in observers:
      stats = []
      for data in newdiff:
          if line == data.split(',')[0]:
              stats.append(float(data.split(',')[3]))
      results[line] = round(np.mean(stats),2)
  totaldiff_file = open('./config-files/Personal-coef.txt','w')
  for key in list(results.keys()):
      totaldiff_file.write(key+'='+str(results[key])+'\n')
      output.append(key+'='+str(results[key]))
  totaldiff_file.close()
  
  return '\n'.join(output)    

def PlotList(obslist, kometa, phasec, forwsca, date_range, magtype, dt = 0):
  xlist, ylist = [], []
  elements = comephem.ephem()
  for line in obslist:
    obs = koprclass.icqcode()
    obs.decode_icq(line)
    eline, deltadate, peridate = kometa.FindElements(obs.date)
    if (dt == 0) or (dt == 1 and deltadate < 0) or (dt == 2 and deltadate > 0):
        elements.readdb(eline)
        if phasec == 1 or forwsca == 1: 
          elements.compute(obs.date,'00:00', ['comet','phasec'])
          obs.mag = obs.mag + elements.phasec * phasec + elements.forwscac * forwsca
        else:
          elements.compute(obs.date,'00:00', ['comet'])
    
        if date_range == 'Date':
          xlist.append(koprhand.timecvt(obs.date))
        elif date_range == 'Days from T':
          xlist.append(deltadate)
        elif date_range == 'r':
          xlist.append(elements.r)
        elif date_range == 'Log(r)':
          xlist.append(log10(elements.r))
    
        if magtype == 1:
          ylist.append(obs.mag - 5*log10(elements.dist))
        else:  
          ylist.append(obs.mag)

  return xlist, ylist     

def PlotListAll(obslist, kometa, phasec, forwsca, date_range, magtype, peri, typ):
  xlist, ylist = [], []
  elements = comephem.ephem()
  for line in obslist:
    obs = koprclass.icqcode()
    obs.decode_icq(line)
    eline, deltadate, peridate = kometa.FindElements(obs.date)
    if peridate[:4] == peri[0] and obs.otype == typ[0]:
        elements.readdb(eline)
        if phasec == 1 or forwsca == 1: 
          elements.compute(obs.date,'00:00', ['comet','phasec'])
          obs.mag = obs.mag + elements.phasec * phasec + elements.forwscac * forwsca
        else:
          elements.compute(obs.date,'00:00', ['comet'])
    
        if date_range == 'Date':
          xlist.append(koprhand.timecvt(obs.date))
        elif date_range == 'Days from T':
          xlist.append(deltadate)
        elif date_range == 'r':
          xlist.append(elements.r)
        elif date_range == 'Log(r)':
          xlist.append(log10(elements.r))
    
        if magtype == 1:
          ylist.append(obs.mag - 5*log10(elements.dist))
        else:  
          ylist.append(obs.mag)

  return xlist, ylist     

def perilen(e, q, peridate):
    if e < 1:
      a = q / ( 1. - e )
      if a < 14:
        perilen = sqrt(a*a*a)*365.25
      else:
        perilen = 19133.
    else:
      perilen = 19133.
    return koprhand.timecvt(peridate) - datetime.timedelta(days = perilen/2.), koprhand.timecvt(peridate) + datetime.timedelta(days = perilen/2.)

def PlotLines(kometa, param, peri, rangefrom, rangeto, date_range, magtype):
  elements, xlist, ylist =  comephem.ephem(), [], []
  eline, deltadate, peridate = kometa.FindElements(peri[0]+'/07/01.00')
  elements.readdb(eline)
  param = param.split(',')
  h0, n, beg, end = float(param[1]), float(param[2]), param[3], param[4]
  
  int1, int2 = perilen(elements.e, elements.q, peridate)
  if beg == 'inf':
     beg = int1
  else:
    beg = koprhand.timecvt(peridate) + datetime.timedelta(days = float(beg))
  if end == 'inf':
     end = int2
  else:
    end = koprhand.timecvt(peridate) + datetime.timedelta(days = float(end))
  
  if date_range == 'Date':
    if beg < rangefrom:
      beg = rangefrom
    if end > rangeto:
      end = rangeto
  elif date_range == 'Days from T':
    if beg < (koprhand.timecvt(peridate) + datetime.timedelta(days = rangefrom)):
      beg = (koprhand.timecvt(peridate) + datetime.timedelta(days = rangefrom))
    if end > (koprhand.timecvt(peridate) + datetime.timedelta(days = rangeto)):
      end = (koprhand.timecvt(peridate) + datetime.timedelta(days = rangeto))

  steps = int((end - beg).days)

  for k in range(steps):
      actualdate = beg + datetime.timedelta(days= k)
      elements.compute(str(actualdate.strftime('%Y/%m/%d')),str(actualdate.strftime('%H:%M')), ['comet'])
      deltat = float((actualdate - koprhand.timecvt(peridate)).days) + float((actualdate - koprhand.timecvt(peridate)).seconds)/86400
      mag = CalcMag(actualdate,eline,float(h0),float(n), magtype)
      
      if date_range == 'Date':
          xlist.append(actualdate)
          ylist.append(mag)
      elif date_range == 'Days from T':
          xlist.append( deltat )
          ylist.append(mag)
      elif date_range == 'r':
          if rangefrom < elements.r < rangeto:
            xlist.append(elements.r)
            ylist.append(mag)
      elif date_range == 'Log(r)':
          if rangefrom < log10(elements.r) < rangeto:
            xlist.append( log10(elements.r) )
            ylist.append(mag)
  return xlist, ylist

def GenerateLines( finallist, excluded, settings, perihelions, types, kometa, rangefrom, rangeto, date_range, magtype, compare, adit ):
  phasec, forwsca, perasy = settings[6], settings[7], settings[4]    
  # colors are stored in perihelions # shapes are stored in types
  points = [] #generate points
  if compare == 1: ##compare
    xlist, ylist = PlotList(excluded, kometa, phasec, forwsca, date_range, magtype)
    points.append([xlist, ylist, 'x', 'red'])  
    xlist, ylist = PlotList(finallist, kometa, phasec, forwsca, date_range, magtype)
    points.append([xlist, ylist, 's', 'black']) 
  elif perasy == 1:    
    xlist, ylist = PlotList(finallist, kometa, phasec, forwsca, date_range, magtype, 1)
    points.append([xlist, ylist, 'x', 'red'])  
    xlist, ylist = PlotList(finallist, kometa, phasec, forwsca, date_range, magtype, 2)
    points.append([xlist, ylist, 's', 'black'])
  else:  
    for peri in perihelions:
      for typ in types:
        xlist, ylist = PlotListAll(finallist, kometa, phasec, forwsca, date_range, magtype, peri, typ )
        points.append([xlist, ylist, typ[2].split(',')[0], peri[2]])

  lines, additparam = [], []#lines
  linelist = [_f for _f in str(adit).split('\n') if _f]
  if linelist:
    for i in range (len(linelist)):
      linenew = linelist[i].replace(' ','').replace('H0=','').replace('mag','').replace('n=','').replace(']','')
      h0 = linenew.split(',')[0]
      n = linenew.split(',')[1].split('[')[0]
      int1 = linenew.split(',')[1].split('[')[1]
      int2 = linenew.split(',')[2]
      additparam.append(kometa.destig+','+h0+','+n+','+int1+','+int2)
  if perasy == 1:
      for peri in perihelions:
        if peri[1] == 'o' :
            for param in kometa.magparams:
              testparam = param.split(',')
              if float(testparam[3].replace('inf','-1')) < 0 and float(testparam[4].replace('inf','1')) > 0:
                modparam = param.split(',')
                modparam[4] = '0' 
                xlist, ylist = PlotLines(kometa, ','.join(modparam), peri, rangefrom, rangeto, date_range, magtype)
                lines.append( [xlist, ylist, 'red', peri[0]] )
                modparam = param.split(',')
                modparam[3] = '0' 
                xlist, ylist = PlotLines(kometa, ','.join(modparam), peri, rangefrom, rangeto, date_range, magtype)
                lines.append( [xlist, ylist, 'black', peri[0]] )
              elif float(testparam[4].replace('inf','1')) <= 0:  
                xlist, ylist = PlotLines(kometa, param, peri, rangefrom, rangeto, date_range, magtype)
                lines.append( [xlist, ylist, 'red', peri[0]] )
              elif float(testparam[3].replace('inf','-1')) >= 0:  
                xlist, ylist = PlotLines(kometa, param, peri, rangefrom, rangeto, date_range, magtype)
                lines.append( [xlist, ylist,  'black', peri[0]] )
              
            for param in additparam: 
              xlist, ylist = PlotLines(kometa, param, peri, rangefrom, rangeto, date_range, magtype)
              lines.append( [xlist, ylist, 'black', peri[0]] )
  else:    
      for peri in perihelions:
        if peri[1] == 'o' :
            for param in kometa.magparams:
              xlist, ylist = PlotLines(kometa, param, peri, rangefrom, rangeto, date_range, magtype)
              lines.append( [xlist, ylist, peri[2], peri[0]] )
            for param in additparam: 
              xlist, ylist = PlotLines(kometa, param, peri, rangefrom, rangeto, date_range, magtype)
              lines.append( [xlist, ylist, 'black', peri[0]] )
      
  return points, lines

def MeanStar(stars):
  diff = []
  for star in stars:
    if star[0] == 'o':
      diff.append(star[1] - star[2])
  return np.mean(diff)    

def CompareObs(output,obs_lines,destig):
    xpt, ypt = [[],[],[],[],[],[],[]], [[],[],[],[],[],[],[]]
    #       x , y
    # [0] date, mym1
    # [1] date, ccdm1
    # [2] date, vism1
    # [3] date, calcm1
    # [4] date, mycoma
    # [5] date, ccdcoma
    # [6] date, viscoma
    xlist, results, stdevs = [ [], [], [], [], [], [], [] ], {}, []
    elements = comephem.ephem()
    kometa = koprclass.comet()
    myobs = koprclass.icqcode()

    kometa.LoadComet(destig)
    myobs.decode_icq(output)
    mydate = koprhand.timecvt(myobs.date)
    xpt[0].append(mydate)
    ypt[0].append(myobs.mag)
    xpt[4].append(mydate)
    ypt[4].append(myobs.coma)
    
    #print myobs.mag, myobs.coma, myobs.dc, myobs.date
    eline = kometa.FindElements(myobs.date)[0]
    elements.readdb(eline)

    for line in obs_lines:
      obs = koprclass.icqcode()
      obs.decode_icq(line)
      date = koprhand.timecvt(obs.date)
      if not obs.negsign == '[':
        #build list of observations to analyze
        elements.compute(obs.date, '00:00', ['comet'] )
        for j in range(7):
            xlist[j].append(float(obs.mag)-5*log10(elements.dist) - (j+2.)*2.5*log10(elements.r))
        #build list to plot  
        if obs.otype == 'ccd':
          xpt[1].append(date)
          ypt[1].append(obs.mag)
          try:
            float(obs.coma)
            xpt[5].append(date)
            ypt[5].append(obs.coma)
          except ValueError:
            pass
          
        elif obs.otype == 'vis':
          xpt[2].append(date)
          ypt[2].append(obs.mag)
          try:
            float(obs.coma)
            xpt[6].append(date)
            ypt[6].append(obs.coma)
          except ValueError:
            pass
    
    if len(xlist[0]) < 3:
      intercept, slope = np.mean(xlist[2]), 4.
    else:
      for j in range(7): # find best parameters
        stdev = np.std(xlist[j])
        stdevs.append(stdev) 
        results[stdev] = [ np.mean(xlist[j]) , j+2. ]
      intercept, slope = results[min(stdevs)]
    xlist, results, stdevs = [ [], [], [], [], [], [], [] ], {}, []    

    steps, datenow = 600, mydate - datetime.timedelta(days = 30)
    for i in range(steps):
        date = str(datenow.strftime('%Y/%m/%d %H:%M'))
        elements.compute(date.split()[0],date.split()[1],['comet'])
        xpt[3].append( datenow )
        ypt[3].append( intercept + 5*log10(elements.dist) + 2.5*slope*log10(elements.r)  )
        datenow = datenow + datetime.timedelta(days = 0.1)  

    return xpt, ypt, intercept, slope

def KkmToPx(rad, pxscale, destig, date, time):
    # calculate comet distance
    comet = comephem.ephem()
    try:
        orb_file = open('./config-files/Elements.txt', 'r')
        for line in orb_file.readlines():
           if line[102:159].startswith(destig):
              comet.readdb(line)
              comet.compute(date,time,['comet'])              
        orb_file.close()
        orb_file = open('./config-files/CustomElements.dat', 'r')
        for line in orb_file.readlines():
           if line[102:159].startswith(destig):
              comet.readdb(line)
              comet.compute(date,time,['comet'])              
        orb_file.close()
    except ValueError:
        pass

    rad, pxscale = float(rad.replace('kkm',''))/149597.8707, abs(float(pxscale))
    # change km to AU     # calculate px radius
    return int(degrees(atan(rad/comet.dist))/pxscale)

def AfrhoCalc(afrho, refstars, extinction, elev, pixszdeg, ddist, rdist, comalt, phasea, code, flt, limit):
    afoutpt = [ [],[],[],[],[],[] ]
    
    sunmag = datavar.GetSunMag(flt)[0]
        
    dyear,dmonth,dday = code.date.split('/')
    dhour = float(code.time.split(':')[0]) + float(code.time.split(':')[0])/60.
    jdate = str(round(alb3.jday(int(dyear),int(dmonth),int(dday),dhour),5))

    afrhores = []
    for key in sorted(afrho):
      rad = tan(radians( key * pixszdeg)) * ddist * 149597870.7
      afmag, error, rest = CalculateMag( [afrho[key][0], comalt], refstars, extinction, elev)
      afoutpt[0].append( rad )
      afoutpt[1].append( afmag )
      afoutpt[2].append( log10( rad ) )
      
      if rad < limit:
          afrhoval = ( 2. * ddist * rdist * 149597870.7 )**2. / rad * 2.512**(sunmag-afmag) * 100000.                
          afrhoerr1 = ( 2. * ddist * rdist * 149597870.7 )**2. / rad * 2.512**(sunmag-afmag-code.error/2.) * 100000.
          afrhoerr2 = ( 2. * ddist * rdist * 149597870.7 )**2. / rad * 2.512**(sunmag-afmag+code.error/2.) * 100000.
          error = 0.5 * ( abs(afrhoval-afrhoerr1) + abs(afrhoval-afrhoerr2) )
           
          if flt != 'C':
            afrhores.append( ', '.join([code.comet, jdate, str(rdist), str(ddist), str(round(phasea,2)), flt, str(int(rad)), str(round(afrhoval,1)), str(round(error,1))]) )
    
          afoutpt[4].append( rad )
          afoutpt[5].append( afrhoval )

    # vypocitat model x, y
    poly = np.poly1d( np.polyfit( afoutpt[2], afoutpt[1], 2 ) )
    der = np.polyder(poly)
    for xrad in afoutpt[2]:
       afoutpt[3].append( poly(xrad) )
      
    limita = der(xrad) 
    
    return afrhores, afoutpt, limita
    
def ClosestStar(x, y, starlist):
    distances, outlist = [], {}
    for star in starlist: # open file with stars x,y
        if ((x-50) < star[0] < (x+50)) and ((y-50) < star[1] < (y+50)): #replace 50 with rad???
            dx, dy = star[0]-x, star[1]-y
            c = dx*dx + dy*dy
            distances.append(c) 
            outlist[c] = star             
    return outlist[min(distances)]

def GetLmSize(lm):       
    if lm < 6: # declare lm
        lm = 6
    elif lm > 16:
        lm = 16
        
    if lm <= 12: # declare catalog
        cat = 'TYC-2'
    if lm > 12:
        cat = 'UCAC4'
                    
    if 14 < lm <= 16: # declare size and length
      size = 1.
    elif 12 < lm <= 14:
      size = 1.5
    elif 10 < lm <= 12:
      size = 2.
    elif 8 < lm <= 10:
      size = 3.
    elif 6 < lm <= 8:
      size = 5.
    elif lm <= 6:
      size = 10.
    
    return lm, cat, size, round(size/0.35,1) 

def GetBortleSurvival(elements, returns):
    if len(returns) == 1:
      eline = next(iter(elements.values()))
      calc = comephem.ephem()
      calc.readdb(eline)
      if calc.e == 1.0:
        return 7.0 + 6.0*calc.q, 1
      else:
        a = calc.q / (1.0-calc.e)
        if a > 100 or a < 0:
          return 7.0 + 6.0*calc.q, 1
        else:
          return 0.0, 0
    else:
      return 0.0, 0

def GetComaXY(header, mapparam):
    try:
      if mapparam[0] == mapparam[5] and mapparam[1] == mapparam[6]:
        return 1024, 768
      else:
        return WorldtoPix(header, float(mapparam[5]), float(mapparam[6]) )
    except:
      return 1024, 768
