#!/usr/bin/python
# -*- coding: utf-8 -*-
#"""
#KOPR 1.0
#"""

import koprhand
import koprfunc
import koprdial
import datavar
from PyQt5 import QtCore, QtGui, QtWidgets
#import matplotlib
#matplotlib.use('Qt4Agg')
import matplotlib.pyplot as plt
import numpy as np

font = datavar.DefaultFont()

## *******************************************
############## WIDGET ################
## *******************************************

class Planner(QtWidgets.QWidget):

    def __init__(self, parent=None):
        super(Planner, self).__init__()
        
        self.initUI()
        
    def initUI(self):               
        self.lecom = ''
        
        koprhand.updatecobs() #load cobs data
#        koprfunc.UpdateComParam() # do cobs update
        comets = koprhand.loadcobsfilecom()

        self.bcom = QtWidgets.QPushButton('Select Comets', self)
        self.bcom.setFont(font)
        self.bcom.clicked.connect(self.CometSelection)

        cobslbl = QtWidgets.QLabel('Actual COBS data and quick analysis: ', self)
        cobslbl.setFont(font)
        self.cbcobs = QtWidgets.QComboBox(self) # comet plot list
        self.cbcobs.setStyleSheet("selection-color: black")
        for item in comets:
            self.cbcobs.addItem(item)
        self.cbcobs.setFont(font)
        self.cbcobs.activated[str].connect(self.onActivated)
        
        grid = QtWidgets.QGridLayout()

        labels = [ 'Start date: ', 'Interval (days): ', 'Steps: ', 'Limit mag:', 'Limit alt.:' ]
        self.elements = []

        for i, name in enumerate(labels):
            label = QtWidgets.QLabel(name)
            label.setFont(font)
            grid.addWidget(label, i, 0 )
            self.elements.append( QtWidgets.QLineEdit(self) )
            self.elements[i].setFont(font)
            self.elements[i].setSizePolicy(QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum))
            grid.addWidget(self.elements[i], i, 1 )
        
        self.cbtwi = QtWidgets.QComboBox(self) 
        self.cbtwi.setStyleSheet("selection-color: black") 
        self.cbtwi.addItem('End of Astronomical twilight')
        self.cbtwi.addItem('End of Nautical twilight')
        self.cbtwi.addItem('End of Civil twilight')
        self.cbtwi.setFont(font)
        self.cbpha = QtWidgets.QCheckBox('Phase correction', self)
        self.cbpha.setFont(font)
        self.cbfws = QtWidgets.QCheckBox('Forward scaterring', self)
        self.cbfws.setFont(font)

        grid.addWidget(self.cbtwi, i+1, 0, 1, 2)
        grid.addWidget(self.cbpha, i+2, 0, 1, 2)
        grid.addWidget(self.cbfws, i+3, 0, 1, 2)
        
        self.beph = QtWidgets.QPushButton('Generate ephemeris', self)
        self.beph.setFont(font)
        self.beph.clicked.connect(self.button_click_ephem)
        self.bdia = QtWidgets.QPushButton('Plot diagram', self)
        self.bdia.setFont(font)
        self.bdia.clicked.connect(self.button_click_diag)
        #self.bmap = QtWidgets.QPushButton('Plot map', self)
        #self.bmap.setFont(font)
        #self.bmap.clicked.connect(self.button_click_map)

        ## set defaults
        self.elements[0].setText(koprhand.DateNow())
        self.elements[1].setText('7')
        self.elements[2].setText('4')
        self.elements[3].setText('15')
        self.elements[4].setText('10')
        
        vboxl = QtWidgets.QVBoxLayout() 
        vboxl.addWidget(self.bcom)
        vboxl.addWidget(cobslbl)
        vboxl.addWidget(self.cbcobs)
        vboxl.addLayout(grid)
        vboxl.addStretch(1)

        self.output = QtWidgets.QTextEdit()
        self.output.setText('')
        self.output.setFont(font)
        self.output.setReadOnly(1)
        self.output.setMinimumSize(600,600)

        btnsave = QtWidgets.QPushButton('Save ephemeris', self)
        btnsave.setFont(font)
        btnsave.clicked.connect(self.button_click_save)

        vboxr = QtWidgets.QVBoxLayout() 
        hbox = QtWidgets.QHBoxLayout()
        hbox.addWidget(self.beph)
        hbox.addWidget(self.bdia)
        hbox.addWidget(btnsave)
        hbox.addStretch(1)
        vboxr.addLayout(hbox)
        vboxr.addWidget(self.output)
        
        hbox = QtWidgets.QHBoxLayout()
        hbox.addLayout(vboxl)
        hbox.addLayout(vboxr)

        self.setLayout(hbox)            
        self.setWindowTitle('KOPR - Observer planner')    
        self.show()        

    def CometSelection(self):
        text, ok = QtWidgets.QInputDialog.getText(self, 'Select comets', 'Input Comets (e.g. 103P or C/2012 X1) separated by commas, none for all: ')
        if ok:
          self.lecom = str(text)

    def button_click_map(self):
        if len(self.lecom.split(',')) != 1:
            koprdial.InfoDialog(1, 'Input one comet', 'You can generate map only for one comet!' )
        else:
            pass
            # generate ephemeris for comet
            lon, lat, elev, gmt, twil, interval, steps, now, comets, maglimit, altlimit, phasec, forwsca = self.Settings()
            ephems = koprhand.MapEphem( lon, lat, elev, gmt, twil, interval, steps, now, comets, maglimit, altlimit, phasec, forwsca )
            magnitudes = [item[3] for item in ephems]
            print('positions =', len(ephems))

            # declare limit magnitude - catalog
            lm, cat, size, leng = koprfunc.GetLmSize( round((max(magnitudes)+2.)*2. , 0 )/2. )

            # select starting ra, ending ra
            startp = (ephems[0][1], ephems[0][2])
            
            #list of positions to plot
            plotlist, i = {}, 0
            
            #while last position is plotted 
            while len(ephems) > 0:
              worklist = sorted(ephems, key=lambda x: x[4])
              plotlist[i] = [] 
              #print 'len ephems', len(ephems) 
              for ephem in worklist:
                #print ephem
                c = ( (startp[0] - ephem[1])**2. + (startp[1] - ephem[2])**2. )**(0.5) #find distance to starting point
                #print c
                plotlist[i].append(ephem)
                if c > leng: #if distance is larger than map length - break but add latest point
                  startp = (ephem[1], ephem[2])
                  #print 'new starting position', startp
                  break
                else:
                  ephems.remove(ephem) 
                  #print 'removed:', ephem
              i += 1
            
            mapinput = []
            for i in range(len(plotlist)):
                #find map center first a predposledni
                print(len(plotlist[i]))
                if len(plotlist[i]) >= 2:          
                  cent = ( np.average([plotlist[i][0][1], plotlist[i][-2][1]]), np.average([plotlist[i][0][2], plotlist[i][-2][2]]) )
                  dra = plotlist[i][0][1] - plotlist[i][-2][1]
                  dde = plotlist[i][0][2] - plotlist[i][-2][2]
                else:
                  cent = ( plotlist[i][0][1], plotlist[i][0][2] )
                #ra, de, fov, maglim, source
                print(cent)  
                header, starlist, vizdata = koprfunc.GenerateMap([ cent[0], cent[1], leng, str(lm), 'UCAC4' ], [], dra, dde )

                pt_lst = [] #get pix coords
                for point in plotlist[i]:
                  pt_lst.append( (koprfunc.WorldtoPix(header, point[1], point[2])) )

                mapinput.append( [ header, starlist, pt_lst, [ cent[0], cent[1], leng, str(lm), 'UCAC4' ], self.lecom ] )

            # plot  
            self.mapplt = PositionPlot(mapinput)
            self.mapplt.exec_()

            # declare number of frames  / length of field
            
    def GenerateMap(self, plotlist):
        pass

    def button_click_ephem(self):
        if self.elements[1].text() == '' or self.elements[2].text() == '' or self.elements[0].text() == '':
            koprdial.InfoDialog(1, 'Errors were encountered', 'Fill all required fields!' )
        else:
            params = koprhand.SettingsRead()
            locname, lon, lat, elev, gmt = params[2]
            twil, interval, steps, now, comets, maglimit, altlimit, phasec, forwsca = self.Settings()
            self.Ephems( float(lon), float(lat), float(elev), float(gmt), twil, interval, steps, now, comets, maglimit, altlimit, phasec, forwsca )

    def Settings(self):
        interval, steps, now = int(self.elements[1].text()), int(self.elements[2].text()), koprhand.timecvt(self.elements[0].text())
        comets = self.lecom
        maglimit, altlimit, phasec, forwsca = 50, -90, 0, 0  

        if self.elements[3].text() != '':
            maglimit = float(self.elements[3].text())
        if self.elements[4].text() != '':
            altlimit = float(self.elements[4].text())
        if self.cbpha.isChecked():
            phasec = 1
        if self.cbfws.isChecked():
            forwsca = 1
        if self.cbtwi.currentIndex() == 0:
            twil = 18.
        elif self.cbtwi.currentIndex() == 1:
            twil = 12.
        elif self.cbtwi.currentIndex() == 2:
            twil = 6.
            
        return twil, interval, steps, now, comets, maglimit, altlimit, phasec, forwsca
        
    def Ephems(self, lon, lat, elev, gmt, twil, interval, steps, now, comets, maglimit, altlimit, phasec, forwsca):
        cometlist = koprhand.EphemGen( lon, lat, elev, gmt, twil, interval, steps, now, comets, maglimit, altlimit, phasec, forwsca )
        self.output.setText(cometlist)
        
    def button_click_save(self):
        fname = QtWidgets.QFileDialog.getSaveFileName( None, 'Save file', './' )
        if fname != '':
            obs_file = open(fname[0], 'w')
            obs_file.write(str(self.output.toPlainText()))
            obs_file.close()

    def button_click_diag(self):
        params = koprhand.SettingsRead()
        locname, lon, lat, elev, gmt = params[2]
        now = koprhand.timecvt(self.elements[0].text())
        comets = self.lecom
        maglimit, altlimit, phasec, forwsca = 50, 5, 0, 0  

        x, y = 1674, 768

        if self.elements[3].text() != '':
            maglimit = float(self.elements[3].text())
        if self.elements[4].text() != '':
            altlimit = float(self.elements[4].text())
        if self.cbpha.isChecked():
            phasec = 1
        if self.cbfws.isChecked():
            forwsca = 1

        colors, colorlist = datavar.Colorlist(), []
        for color in colors:
          if color not in ['darkslategrey']:
            colorlist.append(color)
        cometlist = koprhand.PlotGen( float(lon), float(lat), elev, gmt, 6., now, comets, maglimit, altlimit, phasec, forwsca, now )
        times = koprfunc.GenerateTimes( now.year, now.month, now.day, 0, float(lon), float(lat), int(gmt) )
        t, s = koprfunc.DiagPlotList(now.year, now.month, now.day, float(lon), float(lat), int(gmt), times, cometlist, x, y)

        plt.close('all')
        fig, ax=plt.subplots()
        for i in range(len(cometlist)):
          ax.plot( t[i], s[i], color=colorlist[i], lw = 2, aa = True )
          if t[i][s[i].index(min(s[i]))] < ( x / 6.6 ):
            ax.text( t[i][s[i].index(min(s[i]))], min(s[i]), cometlist[i][0], color=colorlist[i], ha='left', va='bottom', weight='bold' )
          elif t[i][s[i].index(min(s[i]))] > x - ( x / 6.6 ):
            ax.text( t[i][s[i].index(min(s[i]))], min(s[i]), cometlist[i][0], color=colorlist[i], ha='right', va='bottom', weight='bold' )
          else:
            ax.text( t[i][s[i].index(min(s[i]))], min(s[i]), cometlist[i][0], color=colorlist[i], ha='center', va='bottom', weight='bold' )
        diag, ticks = koprfunc.GenerateDiag(times, x, y)
        ax.imshow(diag)
        for tick in ticks:
          ax.vlines( tick[1], 0, 768, color = 'grey', lw = 1 )
          ax.text( tick[1], 768, tick[0], color='grey', ha='center', va='top', weight='bold' )
        ax.text( 0, y/1.5, '30', color='grey', ha='left', va='center', weight='bold' )
        ax.text( 0, y/1.12, '10', color='grey', ha='left', va='center', weight='bold' )
        #plt.legend(bbox_to_anchor=(0., 1., 1., 0.), loc=3, ncol=3, mode="expand", borderaxespad=0.)
        canvas = ax.get_figure().canvas 
        fig.canvas.setWindowTitle('Night diagram '+locname + ' ' + str(now.year) + str(now.month) + str(now.day) + ' (limit mag ' + str(maglimit)+')')
        ax.set_xlim(0,x)
        #ax.get_yaxis().set_tick_params(direction='in')
        ax.get_xaxis().set_ticks([])
        ax.get_yaxis().set_ticks([])
        #plt.axis("off")
        plt.tight_layout()
        plt.show()
    
    def button_click_addcomet(self):
        koprdial.AddComet()

################ comet plot ############################
    def onActivated(self, value): 
        name, magparams, xlist, ylist, t, s = koprhand.GetActualCurve(str(value))
        
        plt.plot(xlist, ylist, 'b+', t, s, 'k') #, 0, intercept, 'bs'
        plt.text(min(xlist), max(ylist)+1.5, magparams)
        plt.title('COBS data plot: '+ name)
        plt.xlabel('Days from now')
        plt.ylabel('Magnitude estimates')
        plt.axis([min(xlist)-5, max(xlist)+35, max(ylist)+2, min(ylist)-2])
        plt.show()

## *******************************************
############## Dialog classes ################
## *******************************************

## Inactive map plot
class PositionPlot(QtWidgets.QDialog):
    def __init__(self, mapinput, parent = None):
        super(PositionPlot, self).__init__()
        vbox = QtWidgets.QVBoxLayout()
        
        self.changpar = False

        print(len(mapinput))
        #x, y = zip(*pt_lst)
        #self.mapplt.sc.pt_plot.set_xdata(x)
        #self.mapplt.sc.pt_plot.set_ydata(y)

        self.sc = koprfig.VisMapDialog2(self, mapinput=mapinput)
        self.mpl_toolbar = NavigationToolbar(self.sc.canvas, self)
        vbox.addWidget(self.mpl_toolbar)
        vbox.addWidget(self.sc)
        self.sc = koprfig.VisMapDialog2(self, mapinput=mapinput)
        vbox.addWidget(self.sc)
        
        btncanc = QtWidgets.QPushButton('Quit', self)
        btncanc.setFont(font)
        btncanc.clicked.connect(self.button_click_cancel)
        
        vbox.addWidget(btncanc)
        
        self.setLayout(vbox)  
        self.setWindowTitle('Map plot')
        self.setWindowModality(QtCore.Qt.ApplicationModal)

    def button_click_cancel(self):
        self.close()

class ScrollPlanner(QtWidgets.QScrollArea):
    """ This scroll area only handles one widget for which scrolling should be provided. """
    def __init__(self, parent=None):
        QtWidgets.QScrollArea.__init__(self, parent)

        self.w = Planner(self)
        self.setWidget(self.w)    
